package com.dxhy.core.controller;

import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import com.dxhy.common.aspect.SysLog;
import com.dxhy.common.constant.CommonConstants;
import com.dxhy.common.controller.AbstractController;
import com.dxhy.common.enums.FplxEnum;
import com.dxhy.common.util.InvoiceUtil;
import com.dxhy.common.util.UserInfoUtil;
import com.dxhy.common.utils.R;
import com.dxhy.core.service.singletable.CoreInvoiceQueryService;

import lombok.extern.slf4j.Slf4j;

/**
 * 发票查验模块接口类
 *
 * @author user
 */

@RestController
@Slf4j
@SuppressWarnings("AlibabaUndefineMagicConstant")
public class InvoiceQueryController extends AbstractController {

    @Resource
    private CoreInvoiceQueryService coreInvoiceQueryService;

    /**
     * 扫码查验发票
     *
     * @return
     *
     */
    @PostMapping("/smcy/info")
    @ResponseBody
    @SysLog("发票扫码查验")
    public ResponseEntity<?> smQueryInvoice(@RequestBody Map<String, String> pramsMap) {
        // 入参统一在入口处理
        String userid = getUserName();
        pramsMap.put("userid", userid);
        String dbName = getUserInfo().getDbName();
        pramsMap.put("dbName", dbName);

        pramsMap.put("company", getUserInfo().getCompany());
        List<String> gfshAll = UserInfoUtil.getGfshAll(getUserInfo().getOrg());
        if (gfshAll.size() > 0) {
            pramsMap.put("taxNo", gfshAll.get(0));

        }
        InvoiceUtil iu = new InvoiceUtil(pramsMap.get("invoiceCode"));
        if(FplxEnum.QDZZP.getFplxDm().equals(pramsMap.get("invoiceType"))
                || FplxEnum.QDPP.getFplxDm().equals(pramsMap.get("invoiceType"))){
            pramsMap.put("invoiceType", pramsMap.get("invoiceType"));
        }else {
            if (pramsMap.get("invoiceCode").isEmpty()) {

                return ResponseEntity.ok(R.error("发票代码不能为空！"));
            }
            pramsMap.put("invoiceType", iu.getFplxdm());
        }


        if (pramsMap.get("invoiceNo").isEmpty()) {
            return ResponseEntity.ok(R.error("发票号码不能为空！"));
        }

        if (pramsMap.get("saveHistory") == null || pramsMap.get("saveHistory").isEmpty()) {
            pramsMap.put("saveHistory", "Y");
        }

        if (!pramsMap.get("invoiceDate").isEmpty()) {
            iu.setKprq(pramsMap.get("invoiceDate"));
            if (!iu.kprqValid(null)) {
                pramsMap.put("invoiceDate", "开票日期格式错误！");
            }
        } else {
            return ResponseEntity.ok(R.error("开票日期不能为空！"));
        }

        if (FplxEnum.ZP.getFplxDm().equals(pramsMap.get("invoiceType"))
                || FplxEnum.JDC.getFplxDm().equals(pramsMap.get("invoiceType"))
                || FplxEnum.DZZP.getFplxDm().equals(pramsMap.get("invoiceType"))
                || FplxEnum.QDZZP.getFplxDm().equals(pramsMap.get("invoiceType"))
                || FplxEnum.QDPP.getFplxDm().equals(pramsMap.get("invoiceType"))) {
            if (pramsMap.get("invoiceAmount").isEmpty()) {
                return ResponseEntity.ok(R.error("金额不能为空！"));
            }
        } else {
            if (pramsMap.get("checkCode").isEmpty()) {
                return ResponseEntity.ok(R.error("校验码不能为空！"));
            }
        }

        try {
            return ResponseEntity.ok(R.ok().put("data", coreInvoiceQueryService.smQueryInvoice(pramsMap)));
        } catch (Exception e) {
            e.printStackTrace();
            log.error("", e);
            return ResponseEntity.ok(R.error(CommonConstants.MSG_ERR_DEFAULT));
        }
    }

}
