package com.dxhy.erp.dao;

import com.dxhy.erp.entity.sdny.BxwxEnterpriseUserEntity;
import com.dxhy.erp.entity.sdny.OrgUserInfo;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Optional;

/**
 * 员工信息
 *
 * @author guoker
 * @date 2020-12-29
 */
public interface OrgUserInfoDao {

    /**
     * 新增员工信息
     *
     * @param orgUserInfoList
     * @return
     */
    int addUserInfoList(@Param("orgUserInfoList") List<OrgUserInfo> orgUserInfoList);

    /**
     * 删除员工信息
     *
     * @param orgUserInfo
     * @return
     */
    int deleteUserInfo(@Param("orgUserInfo") OrgUserInfo orgUserInfo);

    /**
     * 更新员工信息（可批量）
     *
     * @param orgUserInfoList
     * @return
     */
    int updateUserInfo(@Param("orgUserInfoList") List<OrgUserInfo> orgUserInfoList);

    /**
     * 查询员工信息
     *
     * @param userId
     * @param enterpriseId
     * @param taxNo
     * @param identityNumber
     * @param userName
     * @param isWhiteList
     * @param email
     * @return
     */
    List<OrgUserInfo> getUserInfoList(@Param("userId") String userId,
                                      @Param("enterpriseId") String enterpriseId,
                                      @Param("taxNo") String taxNo,
                                      @Param("identityNumber") String identityNumber,
                                      @Param("userName") String userName,
                                      @Param("isWhiteList") String isWhiteList,
                                      @Param("email") String email
    );

    /**
     * 身份证号模糊匹配
     *
     * @param identityNumber
     * @param userName
     * @param enterpriseId
     * @return
     */
    List<OrgUserInfo> getUserIdentityNumber(@Param("identityNumber") String identityNumber, @Param("userName") String userName, @Param("enterpriseId") String enterpriseId);

    /**
     * 身份证号精确匹配统计
     *
     * @param identityNumber
     * @param userName
     * @param enterpriseId
     * @return
     */
    int countUserByIdentityNumber(@Param("identityNumber") String identityNumber, @Param("userName") String userName, @Param("enterpriseId") String enterpriseId);

    /**
     * 根据用户姓名、身份证号码后六位查询完整身份证号码
     *
     * @param userName
     * @param fourId
     * @param enterpriseId
     * @return
     */
    String getIdentityNumberByNameAndFourId(@Param("userName") String userName, @Param("fourID") String fourId, @Param("enterpriseId") String enterpriseId);

    /**
     * 查询总条数
     *
     * @param orgUserInfo
     * @return
     */
    int selectCount(@Param("orgUserInfo") OrgUserInfo orgUserInfo);

    /**
     * 根据类型查询用户集合
     *
     * @param orgUserInfo
     * @param page
     * @param rows
     * @return
     */
    List<OrgUserInfo> selectWhiteByParam(@Param("orgUserInfo") OrgUserInfo orgUserInfo, @Param("page") int page, @Param("rows") int rows);

    /**
     * 修改
     *
     * @param orgUserInfo
     * @return
     */
    int updateWhiteUserById(OrgUserInfo orgUserInfo);

    /**
     * 修改用户白名单状态
     *
     * @param id
     * @return
     */
    int disableWhiteById(@Param("id") String id);

    /**
     * 修改用户授权状态
     *
     * @param userId
     * @param authStatus
     * @return
     */
    int updateAuthstatus(@Param("userId") String userId, @Param("authStatus") String authStatus);

    /**
     * 获取所有白名单用户数据
     *
     * @return
     */
    List<OrgUserInfo> getAllUserInfoList();

    /**
     * 根据id更新微信公众号同步标识
     *
     * @param idList
     * @return
     */
    int batchUpdateEnterpriseUserList(List<String> idList);


    /**
     * 根据id更新微信公众号同步标识和唯一标识id
     *
     * @param bxwxEnterpriseUserEntityList
     * @return
     */
    int batchUpdateEnterpriseUserListById(List<BxwxEnterpriseUserEntity> bxwxEnterpriseUserEntityList);

    /**
     * 新增用户
     *
     * @param orgUserInfo
     * @return
     */
    int saveUserInfo( OrgUserInfo orgUserInfo);

    /**
     * 更新用户
     *
     * @param orgUserInfo
     * @return
     */
    int updateUserInfoSingle( OrgUserInfo orgUserInfo);

    /**
     * 查询员工
     *
     * @param userName
     * @param isWhiteList
     * @return
     */
    List<OrgUserInfo> getUserInfoListByIsWhile( @Param("userName") String userName, @Param("isWhiteList") String isWhiteList);


    /**
     * 根据用户名查询用户信息（精确查询）
     *
     * @param userName
     * @return
     */
    List<OrgUserInfo> queryUserListByUserName(@Param("userName") String userName);


    /**
     * 根据userId查询用户是否存在
     * @param userId userId
     * @return
     */
    Optional<OrgUserInfo> findByCondition(@Param("userId")String userId);
}
