package com.dxhy.erp.service;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSONObject;
import com.dxhy.common.constant.DbConstant;
import com.dxhy.common.datasource.config.DynamicContextHolder;
import com.dxhy.common.util.UserInfoUtil;
import com.dxhy.common.vo.UserInfo;
import com.dxhy.erp.dao.AreaDao;
import com.dxhy.erp.dao.OrgUserInfoDao;
import com.dxhy.erp.entity.sdny.InvoiceDetail;
import com.dxhy.erp.entity.sdny.InvoiceInfo;
import com.dxhy.erp.entity.sdny.area.Area;
import com.dxhy.erp.utils.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 与山东能源交互数据主类
 *
 * @author JLY
 * @date 2017年6月26日下午5:02:49
 */
@SuppressWarnings("AlibabaUndefineMagicConstant")
@Service
@Slf4j
public class SNPushCheckRecordService  extends AbstractServiceAdapter {

    @Value("${sdny.snYxUrl}")
    private String snYxUrl;

    private InvoiceQueryUtil invoiceQueryUtil;

    @Resource
    private AreaDao areaDao;

    @Resource
    private OrgUserInfoDao orgUserInfoDao;

    public String pushCheckResultToSAP(JSONObject result) throws Exception {
        result.put("systemSign", "");
        result.put("roleIds", "");
        log.info("获取角色请求参数：{} , 请求地址：{}", result, snYxUrl);
        String post = HttpUtil.post(snYxUrl, result, 30000);
        log.info("获取角色返回信息：{}", post);
        if (StringUtils.isNotBlank(post)) {
        }

        return post;
    }


    public JSONObject checkInvoice(Map<String, String> pramsMap, UserInfo userInfo) {

        // uuid = 发票代码+发票号码
        String uuid = (org.apache.commons.lang.StringUtils.isBlank(pramsMap.get("invoiceCode"))?"":pramsMap.get("invoiceCode")) + pramsMap.get("invoiceNo");

        DynamicContextHolder.push(userInfo.getDbName() + DbConstant.BUSINESS_READ);

        List<String> gfshAll = UserInfoUtil.getGfshAll(userInfo.getOrg());
        String gfsh = "";
        if (gfshAll.size() > 0) {
            gfsh = gfshAll.get(0);
        }
        JSONObject queryInvoiceResult = invoiceQueryUtil.queryInvoice(pramsMap.get("invoiceCode"),
                pramsMap.get("invoiceNo"), pramsMap.get("invoiceDate"), pramsMap.get("invoiceAmount"), pramsMap.get("checkCode"),
                gfsh,"31");
        String resultCode = queryInvoiceResult.getString("resultCode");

        DynamicContextHolder.push(userInfo.getDbName() + DbConstant.BUSINESS_WRITE);
        // 调用查验接口失败，将签收状态更新到扫描表中或者insert数据到扫描表，然后return；
        if (queryInvoiceResult.containsKey("returnCode")) {
            log.error("发票号码：" + pramsMap.get("invoiceNo") + "签收失败，失败原因:" + queryInvoiceResult.getString("returnMessage"));
            return queryInvoiceResult;
        }
        if (!"0001".equals(resultCode)) {
            // 接口通过但，调用返回失败
            log.error("发票号码：" + pramsMap.get("invoiceNo") + "签收失败，失败原因:" + queryInvoiceResult.getString("resultTip"));
            return queryInvoiceResult;
        }
        return queryInvoiceResult;
    }


    /**
     * 客票价税计算
     *
     * @return
     * @throws ParseException
     */
    public ServiceResult<InvoiceInfo> valoremTaxCount(InvoiceInfo invoiceInfo, String enterpriseId, String isDeductible)
            throws ParseException {
        if (isDeductible == null || StringUtils.isEmpty(isDeductible)) {
            logger.error("客票价税计算失败：费用类别或可抵扣标识为空");
            return fail("费用类别或可抵扣标识为空");
        }
        if (invoiceInfo == null) {
            logger.error("客票价税计算失败：请求参数为空");
            return fail("请求参数为空");
        } else if (StringUtils.isBlank(isDeductible) || !CommonEnum.Y.equals(isDeductible)) {
            return fail("客票不抵扣");
            // 判断发票类型是否为客票
        } else if (!getPassengerInvoiceTypeList().contains(invoiceInfo.getInvoiceTypeCode())) {
            logger.error("客票价税计算失败：发票类型不为客票");
            return fail("发票类型不为客票");
        }

        // 校验客票可抵扣条件
        ServiceResult<?> result = checkIsDeduction(invoiceInfo, enterpriseId);
        if (!result.getResult()) {
            logger.error("客票价税计算失败：" + result.getMsg());
            return fail(result.getMsg());
        }
        // 设置可抵扣标识
        invoiceInfo.setIsDeductible(isDeductible);
        // 设置税率
        // 航空、铁路运输发票税率显示9%
        if (invoiceInfo.getInvoiceTypeCode().equals(Constant.AIR_PASSENGER_ELECTRONIC_TICKET_ITINERARY_CODE)
                || invoiceInfo.getInvoiceTypeCode().equals(Constant.RAILWAY_TICKET_CODE)) {
            invoiceInfo.setTaxRate("9%");
            // 公路、水路及其他显示3% 增值税电子普通发票显示税率字段
        } else if (!invoiceInfo.getInvoiceTypeCode().equals(Constant.VALUE_ADDED_TAX_INVOICE_ELECTRONIC_CODE)
                && !invoiceInfo.getInvoiceTypeCode().equals(Constant.VALUE_ADDED_TAX_INVOICE_ELECTRONIC_PASSENGER_CODE)) {
            invoiceInfo.setTaxRate("3%");
        }

        // 计算税额
        // 航空客票：税额=（票价+燃油附加费）÷（1+9%）×9%
        if (invoiceInfo.getInvoiceTypeCode().equals(Constant.AIR_PASSENGER_ELECTRONIC_TICKET_ITINERARY_CODE)) {
            if (!StringUtils.isNotEmpty(invoiceInfo.getFare()) && isBigDecimal(invoiceInfo.getFare())
                    && StringUtils.isNotEmpty(invoiceInfo.getFuelSurcharge())
                    && isBigDecimal(invoiceInfo.getFuelSurcharge())) {
                // 票价
                BigDecimal fare = new BigDecimal(invoiceInfo.getFare());
                // 燃油附加费
                BigDecimal fuelSurcharge = new BigDecimal(invoiceInfo.getFuelSurcharge());
                invoiceInfo.setInvoiceAmount((fare.add(fuelSurcharge)
                        .divide(new BigDecimal("1.09"), 6, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal("0.09"))));
            } else {
                logger.error("客票价税计算失败：航空客票 票价或燃油附加费为空");
            }
            // 铁路客票：税额=票面金额÷（1+9%）×9%
        } else if (invoiceInfo.getInvoiceTypeCode().equals(Constant.RAILWAY_TICKET_CODE)) {
            invoiceInfo.setTaxAmount(invoiceInfo.getInvoiceAmount()
                    .divide(new BigDecimal("1.09"), 6, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal("0.09")));
            // 公路、水路、其他客票：税额=票面金额÷（1+3%）×3%
        } else if (!invoiceInfo.getInvoiceTypeCode().equals(Constant.VALUE_ADDED_TAX_INVOICE_ELECTRONIC_CODE)
                && !invoiceInfo.getInvoiceTypeCode().equals(Constant.VALUE_ADDED_TAX_INVOICE_ELECTRONIC_PASSENGER_CODE)) {
            invoiceInfo.setInvoiceAmount(invoiceInfo.getInvoiceAmount()
                    .divide(new BigDecimal("1.03"), 6, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal("0.03")));
        }
        // 税额保留两位小数(四舍五入)
        if (invoiceInfo.getInvoiceAmount() != null) {
            invoiceInfo.setTaxAmount(invoiceInfo.getInvoiceAmount().setScale(2, BigDecimal.ROUND_HALF_UP));
        }
        if (!invoiceInfo.getInvoiceTypeCode().equals(Constant.AIR_PASSENGER_ELECTRONIC_TICKET_ITINERARY_CODE)) {
            // 不含税金额
            if (invoiceInfo.getInvoiceAmount() != null && invoiceInfo.getTaxAmount() != null) {
                invoiceInfo.setInvoiceAmount(invoiceInfo.getInvoiceAmount().subtract(invoiceInfo.getTaxAmount()));
            }
        } else {
            if (!StringUtils.isNotEmpty(invoiceInfo.getFare()) && isBigDecimal(invoiceInfo.getFare())
                    && StringUtils.isNotEmpty(invoiceInfo.getFuelSurcharge())
                    && isBigDecimal(invoiceInfo.getFuelSurcharge())) {
                // 票价
                BigDecimal fare = new BigDecimal(invoiceInfo.getFare());
                // 燃油附加费
                BigDecimal fuelSurcharge = new BigDecimal(invoiceInfo.getFuelSurcharge());
                invoiceInfo.setInvoiceAmount(fare.add(fuelSurcharge).subtract(invoiceInfo.getTaxAmount()));
            } else {
                logger.error("客票不含税金额失败：航空客票 票价或燃油附加费为空");
            }
        }
        return ok(invoiceInfo);
    }

    public static List<String> getPassengerInvoiceTypeList() {
        // 客票类型
        List<String> typeList = new ArrayList<>();
        // 增值税电子普通发票
        typeList.add(Constant.VALUE_ADDED_TAX_INVOICE_ELECTRONIC_CODE);
        // 增值税电子普通发票(通行费)
        typeList.add(Constant.VALUE_ADDED_TAX_INVOICE_ELECTRONIC_PASSENGER_CODE);
        // 航空客运电子客票行程单
        typeList.add(Constant.AIR_PASSENGER_ELECTRONIC_TICKET_ITINERARY_CODE);
        // 公路运输客票(实名）
        typeList.add(Constant.ROAD_TRANSPORT_TICKET_REAL_NAME_CODE);
        // 水路运输客票(实名)
        typeList.add(Constant.WATER_TRANSPORT_TICKET_REAL_NAME_CODE);
        // 其他运输客票(实名)
        typeList.add(Constant.OTHER_TRANSPORT_TICKET_REAL_NAME_CODE);
        // 火车票
        typeList.add(Constant.RAILWAY_TICKET_CODE);
        return typeList;
    }

    /**
     * @param invoiceInfo
     * @return
     * @throws ParseException
     */
    private ServiceResult<?> checkIsDeduction(InvoiceInfo invoiceInfo, String enterpriseId) throws ParseException {
        try {
            String result = verifyInfo( invoiceInfo);
            if (!CommonEnum.RESPONSE_INTERFACE_STATUS_CODE_SUCCESS.equals(result)) {
                return fail(result);
            }
            if (Constant.VALUE_ADDED_TAX_INVOICE_ELECTRONIC_CODE.equals(invoiceInfo.getInvoiceTypeCode())
                    || Constant.VALUE_ADDED_TAX_INVOICE_ELECTRONIC_PASSENGER_CODE
                    .equals(invoiceInfo.getInvoiceTypeCode())) {
                if (invoiceInfo.getDetailList() != null) {
                    if (invoiceInfo.getDetailList() == null || invoiceInfo.getDetailList().isEmpty()) {
                        return fail("发票明细不可为空");
                    }
                    // 校验增值税电子普通发票
                    String invoiceTypeCode = invoiceInfo.getInvoiceTypeCode();
                    if (invoiceTypeCode.equals(Constant.VALUE_ADDED_TAX_INVOICE_ELECTRONIC_CODE)
                            || invoiceTypeCode.equals(Constant.VALUE_ADDED_TAX_INVOICE_ELECTRONIC_PASSENGER_CODE)) {
                        List<InvoiceDetail> details = invoiceInfo.getDetailList();
                        if (details != null && details.size() > 0) {
                            for (int i = 0, length = details.size(); i < length; i++) {
                                if (!isRightRule(details.get(i).getGoodsName(), details.get(i).getTaxRate())) {
                                    return fail("发票底账明细中第" + i + 1 + "条名称或税率不正确");
                                }
                                if (StringUtils.isNotEmpty(details.get(i).getTaxRate())) {
                                    if (details.get(i).getTaxRate().contains("%")) {
                                        invoiceInfo.setTaxRate(details.get(i).getTaxRate());
                                    } else {
                                        invoiceInfo.setTaxRate(details.get(i).getTaxRate() + "%");
                                    }
                                }
                            }
                        }
                    }
                } else {
                    return fail("未获取到发票明细");
                }

            }
            // 校验公路、水路、其他运输客票(实名),航空客运电子客票行程单,火车票
            if (invoiceInfo.getInvoiceTypeCode().equals(Constant.ROAD_TRANSPORT_TICKET_REAL_NAME_CODE)
                    || invoiceInfo.getInvoiceTypeCode().equals(Constant.WATER_TRANSPORT_TICKET_REAL_NAME_CODE)
                    || invoiceInfo.getInvoiceTypeCode().equals(Constant.OTHER_TRANSPORT_TICKET_REAL_NAME_CODE)
                    || invoiceInfo.getInvoiceTypeCode().equals(Constant.AIR_PASSENGER_ELECTRONIC_TICKET_ITINERARY_CODE)
                    || invoiceInfo.getInvoiceTypeCode().equals(Constant.RAILWAY_TICKET_CODE)) {
                if (StringUtils.isEmpty(invoiceInfo.getIdentityNumber())
                        || StringUtils.isEmpty(invoiceInfo.getPassenger()) || StringUtils.isEmpty(enterpriseId)) {
                    return fail("必填字段为空");
                }
                int count = orgUserInfoDao.countUserByIdentityNumber(invoiceInfo.getIdentityNumber(),
                        invoiceInfo.getPassenger(), enterpriseId);
                if (count == 0) {
                    return fail("未查询到用户信息");
                }
                if (count > 1) {
                    return fail("查询到多个用户");
                }
            }
            return ok();
        } catch (Exception e) {
            logger.error("校验客运发票是否可抵扣异常:" + e.toString());
            return fail("客运发票是否可抵扣校验失败");
        }
    }

    public String verifyInfo(InvoiceInfo invoiceInfo) {
        String result = "0000";
        if (invoiceInfo == null) {
            result = "发票信息不可为空";
        }
        if (invoiceInfo.getInvoiceTypeCode() == null || "".equals(invoiceInfo.getInvoiceTypeCode())) {
            result = "发票类型不可为空";
        }
        if (invoiceInfo.getInvoiceDate() == null || "".equals(invoiceInfo.getInvoiceDate())) {
            result = "开票日期不可为空";
        }
        if (!CommonEnum.INV_ZZS_ELE_PP.equals(invoiceInfo.getInvoiceTypeCode()) && !CommonEnum.INV_ZZS_ELE_TXF.equals(invoiceInfo.getInvoiceTypeCode())) {
            if (invoiceInfo.getIdentityNumber() == null || "".equals(invoiceInfo.getIdentityNumber())) {
                result = "用户证件号不可为空";
            }
            if (invoiceInfo.getPassenger() == null || "".equals(invoiceInfo.getPassenger())) {
                result = "乘车人不可为空";
            }
            if (invoiceInfo.getDepartCity() == null || invoiceInfo.getDepartCity().isEmpty()) {
                result = "出发城市不可为空";
            }
            if (invoiceInfo.getArriveCity() == null || invoiceInfo.getArriveCity().isEmpty()) {
                result = "到达城市不可为空";
            }
        }
        if (invoiceInfo.getInvoiceDate()
                .compareTo(DateUtil.parse("2019-04-01", DatePattern.NORM_DATE_PATTERN)) < 0) {
            result = "开票日期需要晚于2019年4月1日";
        }
        if (!invoiceInfo.getInvoiceTypeCode().equals(Constant.VALUE_ADDED_TAX_INVOICE_ELECTRONIC_CODE)
                && !invoiceInfo.getInvoiceTypeCode()
                .equals(Constant.VALUE_ADDED_TAX_INVOICE_ELECTRONIC_PASSENGER_CODE)) {
            // 港澳台城市
            List<String> hmtList = new ArrayList<>();
            // 香港
            hmtList.add("香港特别行政区");
            hmtList.add("香港");
            // 澳门
            hmtList.add("澳门特别行政区");
            hmtList.add("澳门");
            // 台湾
            hmtList.add("台湾省");
            hmtList.add("台湾");
            // 出发地和到达地是否为非港澳台城市
            List<Area> departCities = areaDao.queryAreaByName(invoiceInfo.getDepartCity());
            List<Area> arriveCities = areaDao.queryAreaByName(invoiceInfo.getArriveCity());
            boolean invHmtListExisted = (StringUtils.isNotEmpty(invoiceInfo.getDepartCity())
                    && hmtList.contains(invoiceInfo.getDepartCity()))
                    || (StringUtils.isNotEmpty(invoiceInfo.getArriveCity())
                    && hmtList.contains(invoiceInfo.getArriveCity()));
            if (departCities == null || departCities.isEmpty() || arriveCities == null || arriveCities.isEmpty()) {
                result = "出发或到达城市不符合标准";
            } else if (invHmtListExisted) {
                result = "出发或到达城市不符合标准";
            }
        }
        return  result;
    }

    /**
     * 增值税电子普通发票验证逻辑
     *
     * @param goodsName
     * @param rate
     * @return
     */
    private Boolean isRightRule(String goodsName, String rate) {
        return (goodsName.contains("*运输服务*")) && (rate.contains("3") || rate.contains("9"));
    }

    public static boolean isBigDecimal(String str) {
        if (str == null || str.trim().length() == 0) {
            return false;
        }
        char[] chars = str.toCharArray();
        int sz = chars.length;
        int i = (chars[0] == '-') ? 1 : 0;
        if (i == sz) {
            return false;
        }

        // 除了负号，第一位不能为'小数点'
        if (chars[i] == CommonEnum.POINT_CHAR) {
            return false;
        }

        boolean radixPoint = false;
        for (; i < sz; i++) {
            if (chars[i] == '.') {
                if (radixPoint) {
                    return false;
                }
                radixPoint = true;
            } else if (!(chars[i] >= '0' && chars[i] <= '9')) {
                return false;
            }
        }
        return true;
    }

}
